function [f1 f2 f3]=GPre(data, t,varargin)

% EM for the semi-parametric inference of the random effect model
%[] = GPre(...) or [ScaleParameters eta] = GPre(...) calculates the MLE
% [lambda k eta MLV] = GPre(...) returns the maximum log-likelihood value
% time(j)is the set of all measurement time points
% degradation(i,j) is the degradation for ith unit at jth time point (if missing, put it as 0)
% [...] = GPre(degradation, time) calculates the ML estimates of these parameters and the maximum log-likelihood value
% [...] = GPre(degradation, time,limit) specifies termination criterion based on likelihood increment
% [...] = GPre(degradation, time,limit,k,r,eta) specifies the initial parameters values 

if (nargin == 0)
    help EMsg_oneobs;
    return
elseif (nargin > 6)
    error('Too Many Inputs')
elseif (nargout > 3)
    error('Allows At most Three Outputs')
else
    [Y X]=size(data);
    [a X1]=size(t);
    if a > 1 | X1~=X
        error('Input Data Size Mismatch');
    end
    
    %initial values of the parameters
    limit=0.001;lambda_pre=1;k_pre=1;eta_pre=1:1:X;
    if nargin==3;
        limit=varargin{1};
    elseif nargin==4
        limit=varargin{1};lambda_pre=varargin{2};
    elseif nargin==5
        limit=varargin{1};lambda_pre=varargin{2};k_pre=varargin{3};
    elseif nargin==6
        limit=varargin{1};lambda_pre=varargin{2};k_pre=varargin{3};eta_pre=varargin{4};
    end
    LFV_pre=-10^10;
%record the last observation time for each unit
    for i=1:1:Y
        j=X;
        while data(i,j)==0
            j=j-1;
        end
        up(i)=j;
    end
%EM iteration        
    NN=100; N=1;
while N<=NN
    N=N+1;
    
    for i=1:1:Y
        vN(i)=(eta_pre(up(i))+k_pre)/(lambda_pre+data(i,up(i)));
        wN(i)=psi(eta_pre(up(i))+k_pre)-log(lambda_pre+data(i,up(i)));
    end
    temp=log(mean(vN))-mean(wN);
    k_now=fzero(@(x) log(x)-psi(x)-temp, [10e-100 10e100]);
    lambda_now=k_now/mean(vN);
    
    for i=1:1:Y
        j=1;
        while data(i,j)==0
            j=j+1;
        end
        if j==1
            Ey(i,1)=log(data(i,1));
        else
            for nnn=1:1:j
                if nnn==1
                    temp=eta_pre(1);
                else
                    temp=eta_pre(nnn)-eta_pre(nnn-1);
                end
                Ey(i,nnn)=psi(temp)-psi(eta_pre(j))+log(data(i,j));
            end
        end
        
        while j<X
            j_next=j+1;
            while data(i,j_next)==0 & j_next<X
                j_next=j_next+1;
            end
            if j_next==X & data(i,j_next)==0
                for nnn=(j+1):1:j_next
                        Ey(i,nnn)=psi(eta_pre(nnn)-eta_pre(nnn-1))-psi(eta_pre(j)+k_pre)+log(lambda_pre+data(i,j));
                end
            elseif j_next-j==1
                Ey(i,j+1)=log(data(i,j+1)-data(i,j));
            else
                for nnn=(j+1):1:j_next
                    temp=eta_pre(nnn)-eta_pre(nnn-1);
                    Ey(i,nnn)=psi(temp)-psi(eta_pre(j_next)-eta_pre(j))+log(data(i,j_next)-data(i,j));
                end
            end
            j=j_next;
        end
    end
    meanEy=mean(Ey);
	for j=1:1:X
		temp=mean(wN)+meanEy(j);
  
		if j==1
			eta_now(j)=invpsi(temp);
        else
			eta_now(j)=eta_now(j-1)+invpsi(temp);
		end
    end
    
    k_pre=k_now;
    lambda_pre=lambda_now;
    eta_pre=eta_now;
    
    LFV=Y*k_now*log(lambda_now)-Y*gammaln(k_now);
    for m=1:1:Y
        ind=up(m);
        it=eta_now(ind);
        temp=data(m,ind);
        LFV=LFV+gammaln(it+k_now)-(it+k_now)*log(lambda_now+temp);
        j=1;
        while data(m,j)==0
              j=j+1;
        end
        eta=eta_now(j);
        temp=data(m,j);
        LFV=LFV+(eta-1)*log(temp)-gammaln(eta);
        while j<X
            j_next=j+1;
            while j_next<=X & data(m,j_next)==0
                j_next=j_next+1;
            end
            if j_next<=X & data(m,j_next)~=0
                eta=eta_now(j_next)-eta_now(j);
                temp=data(m,j_next)-data(m,j);
                LFV=LFV+(eta-1)*log(temp)-gammaln(eta);
            end
            j=j_next;
        end
    end
    if LFV-LFV_pre<=limit
        N=2*NN;
    else
        LFV_pre=LFV;
    end
%     dis=[LFV k_pre lambda_pre]      

end

if nargout==0
    disp('[k lambda]='), disp([k_now 1/lambda_now]);
    disp('shape function='),disp(eta_now)
else
f1=[k_now 1/lambda_now];f2=eta_now; f3=LFV;
end

end

end

function Y=invpsi(X)
% Y = INVPSI(X)
%
% Inverse digamma (psi) function.  The digamma function is the
% derivative of the log gamma function.  This calculates the value
% Y > 0 for a value X such that digamma(Y) = X.
%
% This algorithm is from Paul Fackler:
% http://www4.ncsu.edu/~pfackler/
%
  L = 1;
  Y = exp(X);
  while L > 10e-8
    Y = Y + L*sign(X-psi(Y));
    L = L / 2;
  end
end