function [f1 f2 f3]=GPsimple(data, t,varargin)%ita_ini,k,lamda)

% EM for the semi-parametric inference of the random effect model
%[] = GPsimple(...) or [r eta] = GPsimple(...) calculates the MLE
% [r eta MLV] = GPsimple(...) returns the maximum log-likelihood value
% time(j)is the set of all measurement time points
% degradation(i,j) is the degradation for ith unit at jth time point (if missing, put it as 0)
% [...] = GPsimple(degradation, time) calculates the ML estimates of these parameters and the maximum log-likelihood value
% [...] = GPsimple(degradation, time,limit) specifies termination criterion based on likelihood increment
% [...] = GPsimple(degradation, time,limit,r,eta) specifies the initial parameters values 

if (nargin == 0)
    help EMsg_oneobs;
    return
elseif (nargin > 5)
    error('Too Many Inputs')
elseif (nargout > 3)
    error('Allow At most Three Outputs')
else
    [Y X]=size(data);
    [a X1]=size(t);
    if a > 1 | X1~=X
        error('Input Data Size Mismatch');
    end
    
    %initial values of the parameters
    limit=0.01;u_pre=.2;eta_pre=1:1:X;
    if nargin==3;
        limit=varargin{1};
    elseif nargin==4
        limit=varargin{1};u_pre=varargin{2};
    elseif nargin==5
        limit=varargin{1};u_pre=varargin{2};eta_pre=varargin{3};
    end
    LFV_pre=-10^10;
%record the last observation time for each unit
    for i=1:1:Y
        j=X;
        while data(i,j)==0
            j=j-1;
        end
        up(i)=j;
    end
%EM iteration        
    NN=100; N=1;
while N<=NN
    N=N+1;

    for m=1:1:Y
        ind=up(m);
        y_upper(m)=data(m,ind)+(eta_pre(X)-eta_pre(ind))*u_pre;
    end
    
    for i=1:1:Y
        j=1;
        while data(i,j)==0
            j=j+1;
        end
        if j==1
            Ey(i,1)=log(data(i,1));
        else
            for nnn=1:1:j
                if nnn==1
                    temp=eta_pre(1);
                else
                    temp=eta_pre(nnn)-eta_pre(nnn-1);
                end
                Ey(i,nnn)=psi(temp)-psi(eta_pre(j))+log(data(i,j));
            end
        end
        
        while j<X
            j_next=j+1;
            while data(i,j_next)==0 & j_next<X
                j_next=j_next+1;
            end
            if j_next==X & data(i,j_next)==0
                for nnn=(j+1):1:j_next
                        Ey(i,nnn)=psi(eta_pre(nnn)-eta_pre(nnn-1))+log(u_pre);
                end
            elseif j_next-j==1
                Ey(i,j+1)=log(data(i,j+1)-data(i,j));
            else
                for nnn=(j+1):1:j_next
                    temp=eta_pre(nnn)-eta_pre(nnn-1);
                    Ey(i,nnn)=psi(temp)-psi(eta_pre(j_next)-eta_pre(j))+log(data(i,j_next)-data(i,j));
                end
            end
            j=j_next;
        end
    end
    test=fzero(@(x) myfunc1(Ey,y_upper,x),[10e-300 10e300]);
    u_now=mean(y_upper)/test;
    eta_now(1)=invpsi(mean(Ey(:,1))-log(u_now));
    for m=2:1:X
        eta_now(m)=eta_now(m-1)+invpsi(mean(Ey(:,m))-log(u_now));
    end
   u_pre=u_now;
   eta_pre=eta_now;
   
   LFV=0;
   for m=1:1:Y
       j=1;
       while data(m,j)==0
           j=j+1;
       end
       eta=eta_now(j); temp=data(m,j);
       LFV=LFV+(eta-1)*log(temp)-eta*log(u_now)-gammaln(eta)-temp/u_now;
        while j<X
            j_next=j+1;
            while j_next<=X & data(m,j_next)==0
                j_next=j_next+1;
            end
            if j_next<=X & data(m,j_next)~=0
                eta=eta_now(j_next)-eta_now(j);
                temp=data(m,j_next)-data(m,j);
                LFV=LFV+(eta-1)*log(temp)-eta*log(u_now)-gammaln(eta)-temp/u_now;
            end
            j=j_next;
        end
   end
%    disp=[LFV u_now]
%    LFV-LFV_pre
   if LFV-LFV_pre<=limit
       N=20*NN;
   else
       LFV_pre=LFV;
   end   
   
end
if nargout == 0
    disp('scale parameter=')
disp(1/u_now)
    disp('shape function:')
disp(eta_now)
elseif nargout > 1
    f1=1/u_now;
    f2=eta_now;
    f3=LFV;
end

end

end

function Y=invpsi(X)
% Y = INVPSI(X)
%
% Inverse digamma (psi) function.  The digamma function is the
% derivative of the log gamma function.  This calculates the value
% Y > 0 for a value X such that digamma(Y) = X.
%
% This algorithm is from Paul Fackler:
% http://www4.ncsu.edu/~pfackler/
%
  L = 1;
  Y = exp(X);
  while L > 10e-8
    Y = Y + L*sign(X-psi(Y));
    L = L / 2;
  end
end